import logging

from datahub.ingestion.graph.client import get_default_graph

logger = logging.getLogger(__name__)


MUTATION_QUERY = """
mutation storeExecutionRequestUploadLocation($input: StoreExecutionRequestUploadLocationInput!) {
  storeExecutionRequestUploadLocation(input: $input) {
    success
  }
}
"""


class CloudCopierLocation:
    def send_location(self, bucket: str, base_path: str, execution_id: str) -> None:
        try:
            self._send_location(bucket, base_path, execution_id)
        except Exception:
            # Adding this to avoid this failing for older server versions
            # or OSS where the endpoint is not available
            logger.warning(f"Failed to send location for execution {execution_id}")

    def _send_location(self, bucket: str, base_path: str, execution_id: str) -> None:
        graph = get_default_graph()
        execution_urn = f"urn:li:dataHubExecutionRequest:{execution_id}"
        location = f"s3://{bucket}/{base_path}/executor-logs/executor-logs.tgz"
        logger.info(f"Sending location for execution {execution_id}: {location}")
        result = graph.execute_graphql(
            MUTATION_QUERY,
            variables={
                "input": {"executionRequestUrn": execution_urn, "location": location},
            },
        )
        logger.info(f"Location sent for execution {execution_id}: {result}")



# Copyright 2021 Acryl Data, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from abc import abstractmethod

from acryl.executor.common.config import ConfigModel
from acryl.executor.context.execution_context import ExecutionContext
from acryl.executor.context.executor_context import ExecutorContext

"""
Single Task Config
"""


class TaskConfig(ConfigModel):
    name: str
    type: str
    configs: dict


class TaskError(Exception):
    """An error occurred when executing a task"""


class Task:
    @classmethod
    @abstractmethod
    def create(cls, config: dict, ctx: ExecutorContext) -> "Task":
        pass

    @abstractmethod
    async def execute(self, args: dict, ctx: ExecutionContext) -> None:
        """
        Execute the task, throws TaskError exception on failure
        """

    @abstractmethod
    def close(self) -> None:
        """
        Wraps up the task
        """



# Copyright 2021 Acryl Data, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging
from typing import Optional, Union

from datahub.ingestion.graph.client import DatahubClientConfig, DataHubGraph
from pydantic import BaseModel, ConfigDict, field_validator

from acryl.executor.secret.datahub_secrets_client import DataHubSecretsClient
from acryl.executor.secret.secret_store import SecretStore

logger = logging.getLogger(__name__)


class DataHubSecretStoreConfig(BaseModel):
    graph_client: Optional[DataHubGraph] = None
    graph_client_config: Optional[DatahubClientConfig] = None

    model_config = ConfigDict(arbitrary_types_allowed=True)

    @field_validator("graph_client", mode="after")
    @classmethod
    def check_graph_connection(
        cls, v: Optional[DataHubGraph]
    ) -> Optional[DataHubGraph]:
        if v is not None:
            v.test_connection()
        return v


# An implementation of SecretStore that fetches secrets from DataHub
class DataHubSecretStore(SecretStore):
    # Client for fetching secrets from DataHub GraphQL API
    client: DataHubSecretsClient

    def __init__(self, config):
        # Attempt to establish an outbound connection to DataHub and create a client.
        if config.graph_client is not None:
            self.client = DataHubSecretsClient(graph=config.graph_client)
        elif config.graph_client_config is not None:
            graph = DataHubGraph(config.graph_client_config)
            self.client = DataHubSecretsClient(graph)
        else:
            raise Exception(
                "Invalid configuration provided: unable to construct DataHub Graph Client."
            )

    def get_secret_values(self, secret_names: list[str]) -> dict[str, Union[str, None]]:
        # Fetch the secret from DataHub, using the credentials provided in the configuration.
        # Use the GraphQL API.
        try:
            return self.client.get_secret_values(secret_names)
        except Exception:
            # Failed to resolve secrets, return empty.
            logger.exception(
                f"Caught exception while attempting to fetch secrets from DataHub. Secret names: {secret_names}"
            )
            return {}

    def get_secret_value(self, secret_name: str) -> Union[str, None]:
        secret_value_dict = self.get_secret_values([secret_name])
        return secret_value_dict.get(secret_name)

    def get_id(self) -> str:
        return "datahub"

    def close(self) -> None:
        pass

    @classmethod
    def create(cls, config):
        config = DataHubSecretStoreConfig.model_validate(config)
        return cls(config)



import os
from typing import Optional, Union

DATAHUB_CLOUD_LOG_PATH_ENV_VAR = "DATAHUB_CLOUD_LOG_PATH"
DATAHUB_CLOUD_LOG_BUCKET_ENV_VAR = "DATAHUB_CLOUD_LOG_BUCKET"
DEFAULT_GMS_PAYLOAD_MAX_LENGTH = 15368520


def string_to_bool(string: str) -> bool:
    return string.lower() == "true"


def get_executor_pool_id() -> Optional[str]:
    return os.environ.get("DATAHUB_EXECUTOR_POOL_ID")


def get_payload_max_length() -> int:
    val = os.environ.get("ACRYL_EXECUTOR_GMS_PAYLOAD_MAX_LENGTH")
    return int(val) if val and val.isdigit() else DEFAULT_GMS_PAYLOAD_MAX_LENGTH


def get_bundled_venv_path() -> str:
    return os.environ.get("DATAHUB_BUNDLED_VENV_PATH", "/opt/datahub/venvs")


def get_dependency_resolution_enabled() -> bool:
    return string_to_bool(
        os.environ.get("INGESTION_DEPENDENCY_RESOLUTION_ENABLED", "true")
    )


def get_cloud_log_bucket() -> Union[str, None]:
    return os.environ.get("DATAHUB_CLOUD_LOG_BUCKET")


def get_cloud_log_path() -> str:
    return os.environ.get("DATAHUB_CLOUD_LOG_PATH", "")


def is_datahub_hosted() -> bool:
    return string_to_bool(os.environ.get("DATAHUB_EXECUTOR_INTERNAL_WORKER", "false"))



# Copyright 2021 Acryl Data, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from abc import abstractmethod

from acryl.executor.request.execution_request import ExecutionRequest
from acryl.executor.request.signal_request import SignalRequest
from acryl.executor.result.execution_result import ExecutionResult


class Executor:
    @abstractmethod
    def execute(self, request: ExecutionRequest) -> ExecutionResult:
        """
        Execute a task
        """

    @abstractmethod
    def signal(self, request: SignalRequest) -> None:
        """
        Signal an executing task
        """

    @abstractmethod
    def shutdown(self) -> None:
        """
        Shutdown the executor
        """

    @abstractmethod
    def get_id(self) -> str:
        """
        Retrieve the id of the Executor
        """



# Copyright 2021 Acryl Data, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import json
import logging
import os
import re
import shutil
import subprocess
from collections.abc import Sequence
from typing import Any

import pydantic
import yaml
from datahub.masking.bootstrap import initialize_secret_masking
from datahub.masking.secret_registry import SecretRegistry

from acryl.executor.common.config import PermissiveConfigModel
from acryl.executor.context.execution_context import ExecutionContext
from acryl.executor.context.executor_context import ExecutorContext
from acryl.executor.execution import venv_utils

logger = logging.getLogger(__name__)


class SubProcessTaskUtil:
    MAX_LOG_LINES = 2000

    # The original value is 64kb (https://github.com/python/cpython/blob/7528e2c06c8baf809b56f406bcc50e8436c9647c/Lib/asyncio/streams.py#L23).
    # Increasing it should improve performance.
    SUBPROCESS_BUFFER_SIZE = 2**20  # 1mb

    # GMS / mysql has a 4mb limit on the size of a data packet.
    # Doing 90% of that so we have some buffer.
    MAX_LOG_SIZE_BYTES = int(0.9 * 2**22)  # 90% of 4mb

    # We want to truncate long lines so that we can show more lines in the logs.
    MAX_BYTES_PER_LINE = 2**12  # 4kb

    @staticmethod
    def format_subprocess_error(e: Exception) -> str:
        """
        Extract detailed error message from subprocess exceptions.

        For CalledProcessError, includes captured subprocess output from stderr or output attributes.
        For other exceptions, returns str(e).

        Args:
            e: The exception to format

        Returns:
            Formatted error message with subprocess output if available
        """
        if isinstance(e, subprocess.CalledProcessError):
            base_msg = str(e)

            # Try stderr first (formatted by runner.py), then output
            error_details = getattr(e, "stderr", None) or getattr(e, "output", None)

            if error_details:
                return f"{base_msg}\n\n{error_details}"

        return str(e)

    @staticmethod
    def _format_log_lines(lines: Sequence[str]) -> str:
        text = "".join(lines)

        # Python slices are super permissive on index bounds, so this works.
        text = text[-SubProcessTaskUtil.MAX_LOG_SIZE_BYTES :]

        if len(lines) >= SubProcessTaskUtil.MAX_LOG_LINES:
            # lines is a deque, so len(lines) won't be larger than MAX_LOG_LINES.
            text = f"[earlier logs truncated...]\n{text}"

        return text

    @staticmethod
    def _resolve_secrets(secret_names: list[str], ctx: ExecutorContext) -> dict:
        # Attempt to resolve secret using by checking each configured secret store.
        secret_stores = ctx.get_secret_stores()
        final_secret_values = dict({})

        for secret_store in secret_stores:
            try:
                # Retrieve secret values from the store.
                secret_values_dict = secret_store.get_secret_values(secret_names)
                # Overlay secret values from each store, if not None.
                for secret_name, secret_value in secret_values_dict.items():
                    if secret_value is not None:
                        final_secret_values[secret_name] = secret_value
            except Exception:
                logger.exception(
                    f"Failed to fetch secret values from secret store with id {secret_store.get_id()}"
                )
        return final_secret_values

    @staticmethod
    def _warn_on_bad_secret_value(ctx: ExecutionContext, key: str, val: str) -> None:
        # Log a warning if the value is a valid JSON document (dict or list)
        # to hint AWS Secret Manager users of a wrong type.
        # We only warn for complex structures, not simple scalar values like numbers.
        try:
            parsed = json.loads(val)
            # Only warn if it's a dict or list (actual JSON documents), not scalars
            if isinstance(parsed, (dict, list)):
                ctx.get_report().report_error(
                    f"Secret variable ${{{key}}} appears to contain a JSON document while string is expected. "
                    "If you are using AWS Secret Manager, make sure to pass secret as plain text and not as a key/value pair."
                )
        except Exception:
            pass

    @staticmethod
    def _resolve_recipe(
        recipe: str, execution_ctx: ExecutionContext, executor_ctx: ExecutorContext
    ) -> tuple[dict, list[str], set[str]]:
        # Now attempt to find and replace all secrets inside the recipe.
        secret_pattern = re.compile(r".*?\${(\w+)}.*?")

        resolved_recipe = recipe
        secret_matches = secret_pattern.findall(resolved_recipe)

        # 1. Extract all secrets needing resolved.
        secrets_to_resolve = []
        if secret_matches:
            for match in secret_matches:
                secrets_to_resolve.append(match)

        # 2. Resolve secret values
        secret_values_dict = SubProcessTaskUtil._resolve_secrets(
            secrets_to_resolve, executor_ctx
        )

        # 3. Fall back to os.environ for any secrets not found in secret stores
        # Track which secrets we need to add (vs. already present)
        secrets_to_add_to_environ = set()

        for secret_name in secrets_to_resolve:
            # Check if secret already exists in environment BEFORE we modify it
            was_already_in_environ = secret_name in os.environ

            if (
                secret_name not in secret_values_dict
                or secret_values_dict[secret_name] is None
            ):
                env_value = os.environ.get(secret_name)
                if env_value is not None:
                    logger.info(
                        f"Secret '{secret_name}' not found in secret stores, using value from environment variable"
                    )
                    secret_values_dict[secret_name] = env_value
                    # Don't add to cleanup - it was already in environment
                else:
                    logger.warning(
                        f"Secret '{secret_name}' not found in secret stores or environment, using empty string"
                    )
                    secret_values_dict[secret_name] = ""
                    secrets_to_add_to_environ.add(secret_name)
            else:
                # Secret came from secret store
                # Only add to cleanup if it wasn't already in environment
                if not was_already_in_environ:
                    secrets_to_add_to_environ.add(secret_name)

        # Set environment variables for SecretRegistry and DataHub CLI
        for name, value in secret_values_dict.items():
            if value is not None and name in secrets_to_add_to_environ:
                os.environ[name] = value

        # Ensure environment cleanup on error
        try:
            # Set up secret masking (must happen after env vars are set)
            if secrets_to_resolve:
                try:
                    initialize_secret_masking(force=True)
                    registry = SecretRegistry.get_instance()
                    for secret_name in secrets_to_resolve:
                        secret_value = os.environ.get(secret_name)
                        if secret_value:
                            registry.register_secret(secret_name, secret_value)

                    logger.info(
                        f"Secret masking enabled for {registry.get_count()} secret(s)"
                    )
                except Exception as e:
                    logger.warning(
                        f"Failed to set up secret masking: {e}. Continuing without masking."
                    )

            # Validate secret values and warn on potential issues
            if secret_matches:
                for match in secret_matches:
                    secret_value = secret_values_dict.get(match, "")
                    SubProcessTaskUtil._warn_on_bad_secret_value(
                        execution_ctx, match, secret_value
                    )

            json_recipe = json.loads(resolved_recipe, strict=False)
            json_recipe["run_id"] = execution_ctx.exec_id

            # Return recipe, all secret names, and secrets we added to environment
            return json_recipe, secrets_to_resolve, secrets_to_add_to_environ
        except Exception:
            # Clean up only the secrets we added (not ones already in environment)
            for secret_name in secrets_to_add_to_environ:
                os.environ.pop(secret_name, None)
            raise

    @staticmethod
    def _get_plugin_from_recipe(recipe: dict) -> str:
        # The source type -- ASSUMPTION ALERT: This should always correspond to the plugin name.
        return recipe["source"]["type"]

    @staticmethod
    def _write_recipe_to_file(
        dir_path: str, recipe: dict, file_name: str = "recipe.yml"
    ) -> str:
        # 1. Create directories to the path
        os.makedirs(dir_path, mode=0o777, exist_ok=True)

        # 2. Dump recipe dictionary to a YAML string
        yaml_recipe = yaml.dump(recipe)

        # 3. Write YAML to file
        file_path = dir_path + "/" + file_name
        with open(file_path, "w") as file_handle:
            file_handle.write(yaml_recipe)

        return file_path

    @staticmethod
    def _remove_directory(dir_path: str) -> None:
        shutil.rmtree(dir_path)


class SubProcessRecipeTaskArgs(PermissiveConfigModel):
    recipe: str
    version: str = "latest"

    extra_pip_requirements: list[str] = []
    extra_pip_plugins: list[str] = []
    extra_env_vars: dict = {}

    @pydantic.field_validator(
        "extra_pip_requirements", "extra_pip_plugins", mode="before"
    )
    @classmethod
    def parse_json_list_fields(cls, v: Any) -> list:
        if isinstance(v, str):
            # Handle corner case where UI passes an empty string
            return [] if v == "" else json.loads(v)
        return v

    @pydantic.field_validator("extra_env_vars", mode="before")
    @classmethod
    def parse_json_dict_field(cls, v: Any) -> dict:
        if isinstance(v, str):
            # Handle corner case where UI passes an empty string
            return {} if v == "" else json.loads(v)
        return v

    def get_venv_name(self, plugin: str) -> str:
        """Generate venv name using venv utilities."""
        return venv_utils.get_venv_name(
            plugin=plugin,
            version=self.version,
            extra_pip_requirements=self.extra_pip_requirements,
            extra_pip_plugins=self.extra_pip_plugins,
        )

    def should_use_bundled_venv(self) -> bool:
        """Check if this configuration should use a Bundled (pre-packaged) venv."""
        return venv_utils.should_use_bundled_venv(self.version)

    def get_combined_env_vars(self) -> dict:
        # Combines os.environ and user-provided custom env vars.
        # User's extra_env_vars will override system environment variables to allow
        # users to explicitly configure their ingestion environment.
        # Filter out empty string values from extra_env_vars to prevent them from overriding
        # non-empty system environment variables with empty values
        filtered_extra_vars = {k: v for k, v in self.extra_env_vars.items() if v != ""}
        combined = {
            **os.environ,  # System vars as base
            **filtered_extra_vars,  # User vars override (non-empty only)
        }

        return combined



import logging
from typing import Optional

import boto3

from acryl.executor.cloud_utils.cloud_copier import CloudCopier

logger = logging.getLogger(__name__)


class S3CloudCopier(CloudCopier):
    log = logging.getLogger(__name__)

    def __init__(
        self,
        bucket: str,
        base_path: str,
        aws_access_key_id: Optional[str] = None,
        aws_secret_access_key: Optional[str] = None,
        aws_session_token: Optional[str] = None,
        region_name: Optional[str] = None,
    ) -> None:
        self.bucket = bucket
        self.base_path = base_path

        # Create boto3 session with provided credentials or use default
        self.session = boto3.session.Session(
            aws_access_key_id=aws_access_key_id,
            aws_secret_access_key=aws_secret_access_key,
            aws_session_token=aws_session_token,
            region_name=region_name,
        )

    def upload(self, source_local_file: str, target_cloud_file: str) -> None:
        s3 = self.session.resource("s3")

        key = self.base_path.rstrip("/") + "/" + target_cloud_file.lstrip("/")
        logger.info(
            f"Uploading {source_local_file} to bucket: {self.bucket} and base path {self.base_path} and key {key}"
        )
        s3.meta.client.upload_file(
            Filename=source_local_file,
            Bucket=self.bucket,
            Key=key,
        )



# Copyright 2021 Acryl Data, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging
from typing import Any, Optional

from datahub.ingestion.graph.client import DataHubGraph

from acryl.executor.cloud_utils.env_utils import get_executor_pool_id

logger = logging.getLogger(__name__)

LIST_EXECUTOR_CONFIGS_QUERY = """query listExecutorConfigs {
    listExecutorConfigs {
        total
        executorConfigs {
            region
            executorId
            queueUrl
            accessKeyId
            secretKeyId
            sessionToken
            expiration
        }
    }
}"""


class ExecutorCredentials:
    def __init__(self, graph: DataHubGraph):
        self.graph = graph

    def _parse_executor_configs_response(
        self, response_data: dict[str, Any], executor_pool_id: str
    ) -> Optional[dict[str, Any]]:
        """
        Parse GraphQL response to extract executor credentials.

        Args:
            response_data: Raw GraphQL response data
            executor_pool_id: The executor pool ID to find credentials for

        Returns:
            Dictionary containing AWS credentials if found, None otherwise
        """

        executor_configs = response_data.get("listExecutorConfigs", {}).get(
            "executorConfigs", []
        )

        # Find the matching executor config
        for config in executor_configs:
            if config.get("executorId") == executor_pool_id:
                logger.debug(f"Found credentials for executor pool {executor_pool_id}")
                return {
                    "region": config.get("region"),
                    "access_key_id": config.get("accessKeyId"),
                    "secret_access_key": config.get("secretKeyId"),
                    "session_token": config.get("sessionToken"),
                    "expiration": config.get("expiration"),
                }

        logger.warning(f"No credentials found for executor pool {executor_pool_id}")
        return None

    def get_executor_credentials(self) -> Optional[dict[str, Any]]:
        """
        Fetch executor credentials from DataHub GraphQL API.

        Args:

        Returns:
            Dictionary containing AWS credentials if found, None otherwise
        """
        executor_pool_id = get_executor_pool_id()
        if not executor_pool_id:
            return None

        logger.debug(f"Fetching executor credentials for pool {executor_pool_id}")
        try:
            res_data = self.graph.execute_graphql(
                query=LIST_EXECUTOR_CONFIGS_QUERY, variables={}
            )

            return self._parse_executor_configs_response(res_data, executor_pool_id)

        except Exception as e:
            logger.exception(
                f"Failed to fetch executor credentials for pool {executor_pool_id}: {e}"
            )
            return None



# Copyright 2021 Acryl Data, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# To be published at https://pypi.org/project/acryl-datahub/.
__package_name__ = "acryl-executor"
__version__ = "0.3.5"


def is_dev_mode() -> bool:
    return __version__ == "0.0.0.dev0"


def nice_version_name() -> str:
    if is_dev_mode():
        return "unavailable (installed editable via git)"
    return __version__



# Copyright 2021 Acryl Data, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import json
import os
import time

# this is an integration test of the functionality, basically, manually verified
from datahub.ingestion.api.common import PipelineContext, RecordEnvelope
from datahub.ingestion.api.sink import NoopWriteCallback
from datahub.ingestion.graph.client import DatahubClientConfig, DataHubGraph
from datahub.ingestion.sink.file import FileSink, FileSinkConfig

from acryl.executor.dispatcher.default_dispatcher import DefaultDispatcher
from acryl.executor.execution.reporting_executor import (
    ReportingExecutor,
    ReportingExecutorConfig,
)
from acryl.executor.execution.task import TaskConfig
from acryl.executor.request.execution_request import ExecutionRequest
from acryl.executor.secret.datahub_secret_store import DataHubSecretStoreConfig
from acryl.executor.secret.secret_store import SecretStoreConfig


class FakeDataHubGraph(DataHubGraph):
    def __init__(self, file_name: str = "mcp_logs.json"):
        self.file_name = file_name
        self.file_sink = FileSink(
            PipelineContext(run_id="test"), FileSinkConfig(filename=file_name)
        )

    def test_connection(self) -> None:
        # Return None to match superclass signature
        pass

    def emit_mcp(self, mcp, *, async_flag=None, emit_mode=None, wait_timeout=None):
        self.file_sink.write_record_async(
            RecordEnvelope(record=mcp, metadata={}), write_callback=NoopWriteCallback()
        )

    def close(self):
        self.file_sink.close()


def setup():
    # Main function

    env_map = {
        "HOST_PORT": "http://localhost:8080",
        "DATABASE": "datahub",
        "MYSQL_USERNAME": "datahub",
        "MYSQL_PASSWORD": "datahub",
    }
    for k, v in env_map.items():
        os.environ[k] = v

    fake_client = FakeDataHubGraph()
    # Build and register executors with the dispatcher
    local_task_config = TaskConfig(
        name="RUN_INGEST",
        type="acryl.executor.execution.sub_process_ingestion_task.SubProcessIngestionTask",
        configs=dict({}),
    )
    local_test_connection_task_config = TaskConfig(
        name="TEST_CONNECTION",
        type="acryl.executor.execution.sub_process_test_connection_task.SubProcessTestConnectionTask",
        configs=dict({}),
    )
    local_exec_config = ReportingExecutorConfig(
        id="default",
        task_configs=[local_task_config, local_test_connection_task_config],
        secret_stores=[
            SecretStoreConfig(type="env", config=dict({})),
            SecretStoreConfig(
                type="datahub",
                config=DataHubSecretStoreConfig(
                    graph_client_config=DatahubClientConfig(
                        server="http://localhost:8080",
                        token="eyJhbGciOiJIUzI1NiJ9.eyJhY3RvclR5cGUiOiJVU0VSIiwiYWN0b3JJZCI6ImRhdGFodWIiLCJ0eXBlIjoiUEVSU09OQUwiLCJ2ZXJzaW9uIjoiMSIsImV4cCI6MTY0MjIyMzE5MywianRpIjoiZjIwZjRmMmQtMThlNS00NTRmLTgyMzQtMTg1YzIxZjAwN2QzIiwic3ViIjoiZGF0YWh1YiIsImlzcyI6ImRhdGFodWItbWV0YWRhdGEtc2VydmljZSJ9.e3N4_Z_3QhviYmLW8UXS2KhhcrBIMinDhSjnR05bMbE",
                    )
                ),
            ),
        ],
        graph_client_config=DatahubClientConfig(
            server="http://localhost:8080",
            token="eyJhbGciOiJIUzI1NiJ9.eyJhY3RvclR5cGUiOiJVU0VSIiwiYWN0b3JJZCI6ImRhdGFodWIiLCJ0eXBlIjoiUEVSU09OQUwiLCJ2ZXJzaW9uIjoiMSIsImV4cCI6MTY0MjIyMzE5MywianRpIjoiZjIwZjRmMmQtMThlNS00NTRmLTgyMzQtMTg1YzIxZjAwN2QzIiwic3ViIjoiZGF0YWh1YiIsImlzcyI6ImRhdGFodWItbWV0YWRhdGEtc2VydmljZSJ9.e3N4_Z_3QhviYmLW8UXS2KhhcrBIMinDhSjnR05bMbE",
        ),
        graph_client=fake_client,
        executor_instance_id="test-executor",
        executor_version="1.0.0",
    )
    return (
        local_task_config,
        local_exec_config,
        fake_client,
        local_test_connection_task_config,
    )


if __name__ == "__main__":
    (
        local_task_config,
        local_exec_config,
        fake_client,
        local_test_connection_task_config,
    ) = setup()
    local_executor = ReportingExecutor(local_exec_config)
    dispatcher = DefaultDispatcher([local_executor])

    endpoint = "http://localhost:8080/"  # get from config
    recipe_dictionary = {
        "source": {"type": "mysql", "config": {"filename": "foobar.yaml"}},
        "sink": {
            "type": "datahub-rest",
            "config": {
                "server": "http://localhost:8080",
                "token": "eyJhbGciOiJIUzI1NiJ9.eyJhY3RvclR5cGUiOiJVU0VSIiwiYWN0b3JJZCI6ImRhdGFodWIiLCJ0eXBlIjoiUEVSU09OQUwiLCJ2ZXJzaW9uIjoiMSIsImV4cCI6MTY0MjIyMzE5MywianRpIjoiZjIwZjRmMmQtMThlNS00NTRmLTgyMzQtMTg1YzIxZjAwN2QzIiwic3ViIjoiZGF0YWh1YiIsImlzcyI6ImRhdGFodWItbWV0YWRhdGEtc2VydmljZSJ9.e3N4_Z_3QhviYmLW8UXS2KhhcrBIMinDhSjnR05bMbE",
            },
        },
    }
    args = {"version": "0.8.41.1rc2", "recipe": json.dumps(recipe_dictionary)}

    exec_request = ExecutionRequest(exec_id="1234", name="TEST_CONNECTION", args=args)
    start_time = time.time()
    dispatcher.dispatch(exec_request)

    dispatcher.shutdown()
    fake_client.close()
    # Now kill the process.
    # signal_request = SignalRequest(exec_id="1234", signal="KILL")
    # dispatcher.dispatch_signal(signal_request)



"""
Venv utilities - Static functions for venv name generation and configuration queries.

This module provides utilities for determining venv names and paths without
performing any actual venv creation or management.
"""

import hashlib
from typing import Union

# Version constants
VENV_VERSION_LATEST = "latest"
VENV_VERSION_BUNDLED = "bundled"
VENV_VERSION_NATIVE = "native"


def is_bundled_version(version: str) -> bool:
    """Check if the version is a bundled version."""
    return version == VENV_VERSION_BUNDLED


def should_use_bundled_venv(version: str) -> bool:
    """Determine if bundled venv should be used based on version."""
    return is_bundled_version(version)


def get_venv_name(
    plugin: str,
    version: str,
    extra_pip_requirements: Union[list[str], None] = None,
    extra_pip_plugins: Union[list[str], None] = None,
) -> str:
    """Generate a venv name based on plugin and configuration."""
    if version == VENV_VERSION_BUNDLED:
        return f"{plugin}-bundled"
    # For other versions, use a hash
    suffix = hashlib.sha256()
    suffix.update(version.encode("utf-8"))
    suffix.update(str(extra_pip_requirements or []).encode("utf-8"))
    suffix.update(str(extra_pip_plugins or []).encode("utf-8"))
    return f"{plugin}-{suffix.digest().hex()[:16]}"


def get_venv_path(venv_name: str, tmp_dir: str) -> str:
    """Get venv path based on venv name and temporary directory."""
    if venv_name.endswith("-bundled"):
        return f"/opt/datahub/venvs/{venv_name}"
    return f"{tmp_dir}/venv-{venv_name}"


def should_use_bundled_venv_by_name(venv_name: str) -> bool:
    """Determine if venv should be treated as bundled based on its name."""
    return venv_name.endswith("-bundled")
